#ifdef _CH_
#pragma package <opencv>
#endif

#ifndef _EiC
#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>
#include <sys/time.h>


#include "theora.h"
#include "vorbis/codec.h"
#include "vorbis/vorbisenc.h"
#endif

//OpenCV
IplImage *image = 0;
int videow=320,videoh=240;
CvCapture* capture = 0;
IplImage* frame = 0;
IplImage *YCC = 0,*cY=0,*cCr=0,*cCb=0;


//Theora
FILE *video=NULL;

int video_x=0;
int video_y=0;
int frame_x=0;
int frame_y=0;
int frame_x_offset=0;
int frame_y_offset=0;
int video_hzn=-1;
int video_hzd=-1;
int video_an=-1;
int video_ad=-1;

int video_r=-1;
int video_q=16;
theora_state td;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
int spinner=0;
char *spinascii="|/-\\";
void spinnit(void){
	spinner++;
	if(spinner==4)spinner=0;
	fprintf(stderr,"\r%c",spinascii[spinner]);
}

int fetch_and_process_video(IplImage *cY,IplImage *cCr,IplImage *cCb,ogg_page *videopage,
							ogg_stream_state *to,
							theora_state *td,
							int videoflag){
	/* You'll go to Hell for using static variables */
	static int          state=-1;
	static unsigned char *yuvframe[2];
	unsigned char        *line;
	yuv_buffer          yuv;
	ogg_packet          op;
	int i, e;

	if(state==-1){
		/* initialize the double frame buffer */
		yuvframe[0]=(unsigned char*)malloc(video_x*video_y*3/2);
		yuvframe[1]=(unsigned char*)malloc(video_x*video_y*3/2);

		/* clear initial frame as it may be larger than actual video data */
		/* fill Y plane with 0x10 and UV planes with 0X80, for black data */
		memset(yuvframe[0],0x10,video_x*video_y);
		memset(yuvframe[0]+video_x*video_y,0x80,video_x*video_y/2);
		memset(yuvframe[1],0x10,video_x*video_y);
		memset(yuvframe[1]+video_x*video_y,0x80,video_x*video_y/2);

		state=0;
	}

	/* is there a video page flushed?  If not, work until there is. */
	while(!videoflag)
	{
		spinnit();

		if(ogg_stream_pageout(to,videopage)>0)
			return 1;
		if(ogg_stream_eos(to))
			return 0;

		{
			{
				yuv.y_width=video_x;
				yuv.y_height=video_y;
				yuv.y_stride=video_x;

				yuv.uv_width=video_x/2;
				yuv.uv_height=video_y/2;
				yuv.uv_stride=video_x/2;

				yuv.y= (unsigned char*)cY->imageData;
				yuv.u= (unsigned char*)cCr->imageData;
				yuv.v= (unsigned char*)cCb->imageData;
			}

			theora_encode_YUVin(td,&yuv);

			/* if there's only one frame, it's the last in the stream */
			theora_encode_packetout(td,0,&op);
			ogg_stream_packetin(to,&op);
		}
	}
	return videoflag;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
int InitCV()
{
	capture = cvCaptureFromCAM(0);

	if( !capture )
	{
		fprintf(stderr,"Could not initialize capturing...\n");
		return -1;
	}

	cvNamedWindow( "Capture", 1 );
	//cvNamedWindow( "YCC", 1 );
	//	cvCreateTrackbar( "Vmin", "Ventana", &vmin, 256, 0 );
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
int main(int argc,char *argv[]){
	int c,long_option_index,ret;

	ogg_stream_state to; /* take physical pages, weld into a logical
	                         stream of packets */
	ogg_stream_state vo; /* take physical pages, weld into a logical
	                         stream of packets */
	ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
	ogg_packet       op; /* one raw packet of data for decode */

	theora_state     td;
	theora_info      ti;
	theora_comment   tc;

	vorbis_info      vi; /* struct that stores all the static vorbis bitstream
	                        settings */
	vorbis_comment   vc; /* struct that stores all the user comments */

	vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
	vorbis_block     vb; /* local working space for packet->PCM decode */

	int videoflag=0;
	int akbps=0;
	int vkbps=0;

	ogg_int64_t video_bytesout=0;
	double timebase;


	FILE* outfile;

#ifdef _WIN32
# ifdef THEORA_PERF_DATA
	LARGE_INTEGER start_time;
	LARGE_INTEGER final_time;

	LONGLONG elapsed_ticks;
	LARGE_INTEGER ticks_per_second;

	LONGLONG elapsed_secs;
	LONGLONG elapsed_sec_mod;
	double elapsed_secs_dbl ;
# endif
#endif

#ifdef THEORA_PERF_DATA
# ifdef WIN32
	QueryPerformanceCounter(&start_time);
# endif
#endif


	/* yayness.  Set up Ogg output stream */
	outfile=fopen("salida.ogg","wb");
	srand(time(NULL));
	{
		/* need two inequal serial numbers */
		int serial1, serial2;
		serial1 = rand();
		serial2 = rand();
		if (serial1 == serial2) serial2++;
		ogg_stream_init(&to,serial1);
		ogg_stream_init(&vo,serial2);
	}


	InitCV();
	frame = cvQueryFrame( capture );
	YCC=cvCreateImage( cvGetSize(frame), 8, 3 );
	cY=cvCreateImage( cvGetSize(frame), 8, 1 );
	cCr=cvCreateImage( cvGetSize(frame), 8, 1 );
	cCb=cvCreateImage( cvGetSize(frame), 8, 1 );

	/* Theora has a divisible-by-sixteen restriction for the encoded video size */
	/* scale the frame size up to the nearest /16 and calculate offsets */
	frame_x=frame->width;
	frame_y=frame->height;
	video_x=((frame_x + 15) >>4)<<4;
	video_y=((frame_y + 15) >>4)<<4;
	/* We force the offset to be even.
	   This ensures that the chroma samples align properly with the luma
	    samples. */
	frame_x_offset=((video_x-frame_x)/2)&~1;
	frame_y_offset=((video_y-frame_y)/2)&~1;

	theora_info_init(&ti);
	ti.width=video_x;
	ti.height=video_y;
	ti.frame_width=frame_x;
	ti.frame_height=frame_y;
	ti.offset_x=frame_x_offset;
	ti.offset_y=frame_y_offset;
	ti.fps_numerator=10;
	ti.fps_denominator=1;
	ti.aspect_numerator=1;
	ti.aspect_denominator=1;
	ti.colorspace=OC_CS_UNSPECIFIED;
	ti.pixelformat=OC_PF_420;
	ti.target_bitrate=0;
	ti.quality=30;

	ti.dropframes_p=0;
	ti.quick_p=1;
	ti.keyframe_auto_p=1;
	ti.keyframe_frequency=64;
	ti.keyframe_frequency_force=64;
	ti.keyframe_data_target_bitrate=0;
	ti.keyframe_auto_threshold=80;
	ti.keyframe_mindistance=8;
	ti.noise_sensitivity=1;

	theora_encode_init(&td,&ti);
	theora_info_clear(&ti);


	/* write the bitstream header packets with proper page interleave */

	/* first packet will get its own page automatically */
	theora_encode_header(&td,&op);
	ogg_stream_packetin(&to,&op);
	if(ogg_stream_pageout(&to,&og)!=1){
		fprintf(stderr,"Internal Ogg library error.\n");
		exit(1);
	}
	fwrite(og.header,1,og.header_len,outfile);
	fwrite(og.body,1,og.body_len,outfile);

	/* create the remaining theora headers */
	theora_comment_init(&tc);
	theora_encode_comment(&tc,&op);
	ogg_stream_packetin(&to,&op);
	theora_encode_tables(&td,&op);
	ogg_stream_packetin(&to,&op);


	/* Flush the rest of our headers. This ensures
	   the actual data in each stream will start
	   on a new page, as per spec. */
	while(1)
	{
		int result = ogg_stream_flush(&to,&og);
		if(result<0)
		{
			/* can't get here */
			fprintf(stderr,"Internal Ogg library error.\n");
			exit(1);
		}
		if(result==0)
			break;
		fwrite(og.header,1,og.header_len,outfile);
		fwrite(og.body,1,og.body_len,outfile);
	}

	/* setup complete.  Raw processing loop */
	fprintf(stderr,"Compressing....\n");
	while(1)
	{
		ogg_page videopage;

		frame = cvQueryFrame( capture );
		printf("Nuevo frame\n");
		if( !frame )
			break;

		//Convierte la imagen a YUV
		cvFlip(frame,0,1);
		cvShowImage( "Capture", frame);
		cvFlip(frame,0,0);
		//cvFlip(frame,0,1);
		cvCvtColor( frame, YCC, CV_BGR2YCrCb);
		cvSplit( YCC, cY, cCr, cCb, 0 );

		/* is there a video page flushed?  If not, fetch one if possible */
		videoflag=fetch_and_process_video(cY,cCr,cCb,&videopage,&to,&td,videoflag);

		/* no pages of either?  Must be end of stream. */
		if(!videoflag)
			break;

		/* which is earlier; the end of the audio page or the end of the
		   video page? Flush the earlier to stream */
		{
			int audio_or_video=-1;
			double videotime=
				videoflag?theora_granule_time(&td,ogg_page_granulepos(&videopage)):-1;

			/* flush a video page */
			video_bytesout+=fwrite(videopage.header,1,videopage.header_len,outfile);
			video_bytesout+=fwrite(videopage.body,1,videopage.body_len,outfile);
			videoflag=0;
			timebase=videotime;


			{
				int hundredths=timebase*100-(long)timebase*100;
				int seconds=(long)timebase%60;
				int minutes=((long)timebase/60)%60;
				int hours=(long)timebase/3600;

				if(audio_or_video)
					vkbps=rint(video_bytesout*8./timebase*.001);

				fprintf(stderr,
						"\r      %d:%02d:%02d.%02d video: %dkbps                 ",
						hours,minutes,seconds,hundredths,vkbps);
			}
		}

		//cvShowImage( "YCC", YCC);
		c = cvWaitKey(10);
		if( c == 27 )
			break;

	}

	/* clear out state */

	if(video)
	{
		ogg_stream_clear(&to);
		theora_clear(&td);
	}

	cvReleaseImage(&YCC);
	cvReleaseImage(&cY);
	cvReleaseImage(&cCr);
	cvReleaseImage(&cCb);

	if(outfile && outfile!=stdout)
		fclose(outfile);

	fprintf(stderr,"\r   \ndone.\n\n");

	return(0);

}
